@@ -436,7 +436,11 @@ get_parent_context_ref (GtkAccessible *accessible)
         gtk_accessible_get_at_context (GTK_ACCESSIBLE (parent));
 
       if (parent_context != NULL)
-        res = gtk_at_spi_context_to_ref (GTK_AT_SPI_CONTEXT (parent_context));
+        {
+          gtk_at_context_realize (parent_context);
+
+          res = gtk_at_spi_context_to_ref (GTK_AT_SPI_CONTEXT (parent_context));
+        }
     }
 
   if (res == NULL)
@@ -949,22 +953,30 @@ gtk_at_spi_context_state_change (GtkATContext                *ctx,
     {
       value = gtk_accessible_attribute_set_get_value (states, GTK_ACCESSIBLE_STATE_CHECKED);
 
-      switch (gtk_tristate_accessible_value_get (value))
+      if (value->value_class->type == GTK_ACCESSIBLE_VALUE_TYPE_TRISTATE)
+        {
+          switch (gtk_tristate_accessible_value_get (value))
+            {
+            case GTK_ACCESSIBLE_TRISTATE_TRUE:
+              emit_state_changed (self, "checked", TRUE);
+              emit_state_changed (self, "indeterminate", FALSE);
+              break;
+            case GTK_ACCESSIBLE_TRISTATE_MIXED:
+              emit_state_changed (self, "checked", FALSE);
+              emit_state_changed (self, "indeterminate", TRUE);
+              break;
+            case GTK_ACCESSIBLE_TRISTATE_FALSE:
+              emit_state_changed (self, "checked", FALSE);
+              emit_state_changed (self, "indeterminate", FALSE);
+              break;
+            default:
+              break;
+            }
+        }
+      else
         {
-        case GTK_ACCESSIBLE_TRISTATE_TRUE:
-          emit_state_changed (self, "checked", TRUE);
-          emit_state_changed (self, "indeterminate", FALSE);
-          break;
-        case GTK_ACCESSIBLE_TRISTATE_MIXED:
           emit_state_changed (self, "checked", FALSE);
           emit_state_changed (self, "indeterminate", TRUE);
-          break;
-        case GTK_ACCESSIBLE_TRISTATE_FALSE:
-          emit_state_changed (self, "checked", FALSE);
-          emit_state_changed (self, "indeterminate", FALSE);
-          break;
-        default:
-          break;
         }
     }
 
@@ -1007,22 +1019,31 @@ gtk_at_spi_context_state_change (GtkATContext                *ctx,
   if (changed_states & GTK_ACCESSIBLE_STATE_CHANGE_PRESSED)
     {
       value = gtk_accessible_attribute_set_get_value (states, GTK_ACCESSIBLE_STATE_PRESSED);
-      switch (gtk_tristate_accessible_value_get (value))
+
+      if (value->value_class->type == GTK_ACCESSIBLE_VALUE_TYPE_TRISTATE)
+        {
+          switch (gtk_tristate_accessible_value_get (value))
+            {
+            case GTK_ACCESSIBLE_TRISTATE_TRUE:
+              emit_state_changed (self, "pressed", TRUE);
+              emit_state_changed (self, "indeterminate", FALSE);
+              break;
+            case GTK_ACCESSIBLE_TRISTATE_MIXED:
+              emit_state_changed (self, "pressed", FALSE);
+              emit_state_changed (self, "indeterminate", TRUE);
+              break;
+            case GTK_ACCESSIBLE_TRISTATE_FALSE:
+              emit_state_changed (self, "pressed", FALSE);
+              emit_state_changed (self, "indeterminate", FALSE);
+              break;
+            default:
+              break;
+            }
+        }
+      else
         {
-        case GTK_ACCESSIBLE_TRISTATE_TRUE:
-          emit_state_changed (self, "pressed", TRUE);
-          emit_state_changed (self, "indeterminate", FALSE);
-          break;
-        case GTK_ACCESSIBLE_TRISTATE_MIXED:
           emit_state_changed (self, "pressed", FALSE);
           emit_state_changed (self, "indeterminate", TRUE);
-          break;
-        case GTK_ACCESSIBLE_TRISTATE_FALSE:
-          emit_state_changed (self, "pressed", FALSE);
-          emit_state_changed (self, "indeterminate", FALSE);
-          break;
-        default:
-          break;
         }
     }
 
